package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.api.service.HandoverOperationApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabHandoverExternal;
import com.gic.haoban.manage.service.entity.TabHandoverStaff;
import com.gic.haoban.manage.service.entity.TabHandoverTransfer;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.service.*;
import com.gic.wechat.api.dto.qywx.QywxTransferCustomerInfoDTO;
import com.gic.wechat.api.dto.qywx.QywxUnassignedInfoDTO;
import com.gic.wechat.api.dto.qywx.response.QywxGetUnassignedListDTO;
import com.gic.wechat.api.dto.qywx.response.QywxTransferCustomerDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * Created 2021/6/16.
 *
 * @author hua
 */
@Service
public class HandoverOperationApiServiceImpl implements HandoverOperationApiService {

    private static final Logger logger = getLogger(HandoverOperationApiServiceImpl.class);

    @Autowired
    private HandoverService handoverService;

    @Autowired
    private QywxUserApiService qywxUserApiService;

    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Autowired
    private Config config;

    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;


    @Autowired
    private StaffService staffService;


    @Override
    public void batchQywxHandoverTimeTask(String params) {
        logger.info("定时拉取离职基础的人员：{}", params);
        List<YwWxEnterpriseDTO> dtoList = wxEnterpriseApiService.listAll();
        for (YwWxEnterpriseDTO enterpriseDTO : dtoList) {
            try {
                dealQywxEnterpriseHandover(enterpriseDTO.getWxEnterpriseId());
                dealQywxEnterpriseHandoverResult(enterpriseDTO.getWxEnterpriseId());
            } catch (Exception e) {
                e.printStackTrace();
                logger.info("商户处理异常：{},{}", JSONObject.toJSONString(enterpriseDTO), e);
            }
        }
    }

    @Override
    public void BatchQywxHandoverResultTimeTask(String params) {
        logger.info("离职基础结果开始：{}", params);
        List<YwWxEnterpriseDTO> dtoList = wxEnterpriseApiService.listAll();
        for (YwWxEnterpriseDTO enterpriseDTO : dtoList) {
            try {
                dealQywxEnterpriseHandoverResult(enterpriseDTO.getWxEnterpriseId());
            } catch (Exception e) {
                e.printStackTrace();
                logger.info("商户处理异常：{},{}", JSONObject.toJSONString(enterpriseDTO), e);
            }
        }
    }

    @Override
    public void dealQywxEnterpriseHandover(String wxEnterpriseId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        logger.info("拉取单个企业微信的离职成员：{}", wxEnterpriseId, JSONObject.toJSONString(wxEnterpriseDTO));
        List<QywxUnassignedInfoDTO> allQywxUnassigned = getAllQywxUnassigned(wxEnterpriseDTO);
        logger.info("处理离职成员数量：{},{}", wxEnterpriseId, allQywxUnassigned.size());
        handoverService.saveHandover(wxEnterpriseId, allQywxUnassigned);
    }

    private List<QywxUnassignedInfoDTO> getAllQywxUnassigned(WxEnterpriseDTO wxEnterpriseDTO) {
        List<QywxUnassignedInfoDTO> ret = new ArrayList<>();
        QywxGetUnassignedListDTO unassignedListDTO = null;
        String cursor = null;
        do {
            unassignedListDTO = qywxUserApiService.getUnassignedList(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), cursor);
            cursor = unassignedListDTO.getNextCursor();
            if (CollectionUtils.isNotEmpty(unassignedListDTO.getInfo())) {
                ret.addAll(unassignedListDTO.getInfo());
            }
        } while (StringUtils.isNotBlank(cursor) && unassignedListDTO.getErrcode() == 0);
        return ret;
    }


    @Override
    public void dealQywxEnterpriseHandoverResult(String wxEnterpriseId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        Date date = DateUtils.addDays(new Date(), -3);
        String day = DateFormatUtils.format(date, "yyyy-MM-dd");
        List<TabHandoverTransfer> transfers = handoverService.listTransferByWxEnterpriseId(wxEnterpriseId, day);
        if (CollectionUtils.isEmpty(transfers)) {
            logger.info("没有需要转化的");
            return;
        }
        transfers.forEach(dto -> {
            try {
                TabHaobanStaff handover = staffService.selectById(dto.getHandoverStaffId());
                TabHaobanStaff takeover = staffService.selectById(dto.getTakeoverStaffId());
                QywxTransferCustomerDTO customerDTO = qywxUserApiService.transferResult(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), handover.getWxUserId(), takeover.getWxUserId(), true);
                if (customerDTO.getErrcode() != 0) {
                    return;
                }
                List<QywxTransferCustomerInfoDTO> customerResults = customerDTO.getCustomer();
                handoverService.dealResult(wxEnterpriseId, dto.getHandoverTransferId(), customerResults);

            } catch (Exception e) {
                e.printStackTrace();
                logger.info("处理异常：{},{}", JSONObject.toJSONString(dto), e);
            }
        });
    }

    @Override
    public List<HandoverStaffInfoDTO> listHandoverStaff(String wxEnterpriseId, String storeId) {
        List<TabHandoverStaff> handoverStaffs = handoverService.listByStoreId(wxEnterpriseId, storeId);
        if (CollectionUtils.isEmpty(handoverStaffs)) {
            return null;
        }
        Map<String, TabHandoverStaff> handoverStaffMap = handoverStaffs.stream().collect(Collectors.toMap(dto -> dto.getStaffId(), dto -> dto, (o, n) -> o));

        List<String> staffIds = handoverStaffs.stream().map(dto -> dto.getStaffId()).collect(Collectors.toList());
        List<TabHaobanStaff> tabHaobanStaffs = staffService.listNoStatusByStaffIds(wxEnterpriseId, staffIds);
        if (CollectionUtils.isEmpty(tabHaobanStaffs)) {
            logger.info("不存在staff列表");
            return null;
        }
        List<HandoverStaffInfoDTO> retList = EntityUtil.changeEntityListByJSON(HandoverStaffInfoDTO.class, tabHaobanStaffs);
        retList.forEach(dto -> {
            TabHandoverStaff handoverStaff = handoverStaffMap.get(dto.getStaffId());
            dto.setDimissionTime(handoverStaff.getDimissionTime());
            dto.setFriendCount(handoverStaff.getFriendCount());
            dto.setHaobanFriendCount(handoverStaff.getHaobanFriendCount());
        });
        return retList;
    }

    @Override
    public Page<HandoverExternalDTO> pageHandoverFriend(String wxEnterpriseId, String search, List<String> staffIds, List<String> transferIds, BasePageInfo pageInfo) {
        Page<HandoverExternalDTO> retPage = handoverService.pageByStaffIds(wxEnterpriseId, search, staffIds, transferIds, pageInfo, 1);
        if (CollectionUtils.isEmpty(retPage.getResult())) {
            return retPage;
        }
        List<String> midStaffIds = retPage.getResult().stream().map(dto -> dto.getStaffId()).collect(Collectors.toList());
        List<TabHaobanStaff> haobanStaffs = staffService.listNoStatusByStaffIds(wxEnterpriseId, midStaffIds);
        Map<String, String> staffNameMap = haobanStaffs.stream().collect(Collectors.toMap(dto -> dto.getStaffId(), dto -> dto.getStaffName(), (o, n) -> o));
        retPage.getResult().forEach(dto -> {
            String staffName = staffNameMap.get(dto.getStaffId());
            dto.setStaffName(staffName);
        });
        return retPage;
    }

    @Override
    public Page<ExternalInfoDTO> pageHandoverFriend(String wxEnterpriseId, String search, List<String> staffIds, List<String> transferIds, BasePageInfo pageInfo, int status) {
        Page<HandoverExternalDTO> retPage = handoverService.pageByStaffIds(wxEnterpriseId, search, staffIds, transferIds, pageInfo, status);
        if (CollectionUtils.isEmpty(retPage.getResult())) {
            return new Page<>();
        }
        Page<ExternalInfoDTO> ret = PageUtil.changeToCurrentPage(retPage, ExternalInfoDTO.class);
        List<ExternalInfoDTO> infoDTOS = ret.getResult();

        List<String> retTransferIds = infoDTOS.stream().map(dto -> dto.getHandoverTransferId()).collect(Collectors.toList());
        List<TabHandoverTransfer> transfers = handoverService.listTransferByIds(wxEnterpriseId, retTransferIds);
        Map<String, TabHandoverTransfer> transferMap = transfers.stream().collect(Collectors.toMap(dto -> dto.getHandoverTransferId(), dto -> dto));

        List<String> midStaffIds = transfers.stream().flatMap(dto -> {
            List<String> mid = new ArrayList<>();
            mid.add(dto.getHandoverStaffId());
            mid.add(dto.getTakeoverStaffId());
            return mid.stream();
        }).collect(Collectors.toList());

        List<TabHaobanStaff> haobanStaffs = staffService.listNoStatusByStaffIds(wxEnterpriseId, midStaffIds);
        Map<String, String> staffNameMap = haobanStaffs.stream().collect(Collectors.toMap(dto -> dto.getStaffId(), dto -> dto.getStaffName(), (o, n) -> o));
        ret.getResult().forEach(dto -> {
            TabHandoverTransfer handoverTransfer = transferMap.get(dto.getHandoverTransferId());
            String handoverStaff = staffNameMap.get(handoverTransfer.getHandoverStaffId());
            String takeoverStaff = staffNameMap.get(handoverTransfer.getTakeoverStaffId());
            dto.setTakeoverStaffName(takeoverStaff);
            dto.setHandoverStaffId(handoverTransfer.getHandoverStaffId());
            dto.setTakeoverStaffId(handoverTransfer.getTakeoverStaffId());
            dto.setHandoverStaffName(handoverStaff);
        });
        return ret;
    }

    @Override
    public void transferFriend(String wxEnterpriseId, List<String> handoverExternalIds, String disStaffId, String storeId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        TabHaobanStaff tabHaobanStaff = staffService.selectById(disStaffId);
        List<TabHandoverExternal> externalList = handoverService.listByIds(wxEnterpriseId, handoverExternalIds);
        //分配人与客户的关联map
        Map<String, List<String>> handoverStaffExternalMap = externalList.stream().collect(Collectors.groupingBy(dto -> dto.getUserId(), Collectors.mapping(TabHandoverExternal::getExternalUserId, Collectors.toList())));
        //分配人wxUserId 对应的staffIdMap
        Map<String, String> handoverStaffIdMap = externalList.stream().collect(Collectors.toMap(dto -> dto.getUserId(), dto -> dto.getStaffId(), (o, n) -> o));
        //TabHandoverExternal 根据分配人wxUserId 分类 主键id
        Map<String, List<String>> userByIdMap = externalList.stream().collect(Collectors.groupingBy(dto -> dto.getUserId(), Collectors.mapping(dto -> dto.getHandoverExternalId(), Collectors.toList())));

        handoverStaffExternalMap.forEach((wxUserId, list) -> {
            QywxTransferCustomerDTO transferCustomerResp = qywxUserApiService.transferCustomer(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), wxUserId, tabHaobanStaff.getWxUserId(), list);
            if (transferCustomerResp.getErrcode() != 0) {
                logger.info("转移异常");
            }
            List<QywxTransferCustomerInfoDTO> customer = transferCustomerResp.getCustomer();
            if (CollectionUtils.isEmpty(customer)) {
                return;
            }
            TabHandoverTransfer transfer = new TabHandoverTransfer();
            transfer.setExternalCount(list.size());
            transfer.setStoreId(storeId);
            transfer.setHandoverStaffId(handoverStaffIdMap.get(wxUserId));
            transfer.setTakeoverStaffId(disStaffId);
            transfer.setWxEnterpriseId(wxEnterpriseId);
            //执行保存分配后结果
            handoverService.saveHandoverTransfer(transfer, userByIdMap.get(wxUserId));

        });
    }

    @Override
    public List<HandoverExternalDTO> listByIds(String wxEnterpriseId, List<String> handoverExternalIds) {
        List<TabHandoverExternal> externalList = handoverService.listByIds(wxEnterpriseId, handoverExternalIds);
        return EntityUtil.changeEntityListByJSON(HandoverExternalDTO.class, externalList);
    }

    @Override
    public List<HandoverStaffInfoDTO> listDis(String wxEnterpriseId, String storeId) {
        List<TabHandoverTransfer> handoverTransfers = handoverService.listTransferByStoreId(wxEnterpriseId, storeId);
        if (CollectionUtils.isEmpty(handoverTransfers)) {
            return null;
        }
        Map<String, TabHandoverTransfer> handoverTransferMap = handoverTransfers.stream().collect(Collectors.toMap(dto -> dto.getHandoverStaffId(), dto -> dto, (o, n) -> o));

        List<String> handoverStaffIds = handoverTransfers.stream().map(dto -> dto.getHandoverStaffId()).collect(Collectors.toList());
        List<TabHaobanStaff> haobanStaffs = staffService.listNoStatusByStaffIds(wxEnterpriseId, handoverStaffIds);
        List<HandoverStaffInfoDTO> retList = EntityUtil.changeEntityListByJSON(HandoverStaffInfoDTO.class, haobanStaffs);
        retList.forEach(dto -> {
            TabHandoverTransfer mid = handoverTransferMap.get(dto.getStaffId());
            dto.setFriendCount(mid.getExternalCount());
        });

        return retList;
    }

    @Override
    public List<String> listTransferIdByHandoverStaffId(String wxEnterpriseId, String handoverStaffId) {
        List<TabHandoverTransfer> ret = handoverService.listTransferByHandoverStaffId(wxEnterpriseId, handoverStaffId);
        if (CollectionUtils.isEmpty(ret)) {
            return null;
        }
        return ret.stream().map(dto -> dto.getHandoverTransferId()).collect(Collectors.toList());
    }

    @Override
    public List<String> listTransferIdByTakeoverStaffId(String wxEnterpriseId, String takeoverStaffId) {
        List<TabHandoverTransfer> ret = handoverService.listTransferByTakeoverStaffId(wxEnterpriseId, takeoverStaffId);
        if (CollectionUtils.isEmpty(ret)) {
            return null;
        }
        return ret.stream().map(dto -> dto.getHandoverTransferId()).collect(Collectors.toList());
    }
}
