package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.QywxTagRelationTypeEnum;
import com.gic.haoban.manage.api.service.QywxTagApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.entity.TabQywxTag;
import com.gic.haoban.manage.service.entity.TabQywxTagItem;
import com.gic.haoban.manage.service.entity.TabQywxTagRelation;
import com.gic.haoban.manage.service.service.QywxTagService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.wechat.api.dto.qywx.QywxTagDTO;
import com.gic.wechat.api.dto.qywx.QywxTagGroupDTO;
import com.gic.wechat.api.dto.qywx.response.QywxGetCorpTagListDTO;
import com.gic.wechat.api.dto.qywx.response.QywxResponseDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * Created 2021/6/21.
 *
 * @author hua
 */
@Service
public class QywxTagApiServiceImpl implements QywxTagApiService {
    private static final Logger logger = getLogger(QywxTagApiServiceImpl.class);


    @Autowired
    private QywxUserApiService qywxUserApiService;

    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;

    @Autowired
    private Config config;

    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Autowired
    private QywxTagService qywxTagService;

    @Override
    public void pullQywxTag(String wxEnterpriseId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        if (null == wxEnterpriseDTO) {
            logger.info("企业标签所在企业不存在");
            return;
        }
        QywxGetCorpTagListDTO corpTagListResp = qywxSuiteApiService.getCorpTagList(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), null, -1);
        if (corpTagListResp.getErrcode() != 0) {
            logger.info("企业微信标签有问题:{}", wxEnterpriseId);
            return;
        }
        //拉取保存所有标签
        List<QywxTagGroupDTO> tagGroup = corpTagListResp.getTagGroup();
        for (QywxTagGroupDTO groupDTO : tagGroup) {
            qywxTagService.saveQywxTagByQywxGroupKey(wxEnterpriseId, groupDTO);
        }


    }

    @Override
    public void syncTagToQywx(String wxEnterpriseId, String enterpriseId, QywxTagInfoDTO infoDTO, List<QywxTagItemDTO> items) {

        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        //调用企业微信接口保存
        QywxGetCorpTagListDTO resp = this.saveQywxTag(wxEnterpriseDTO, infoDTO, items);
        if (resp.getErrcode() != 0) {
            logger.info("同步失败：{}", JSONObject.toJSONString(resp));
            return;
        }
        //拉取保存所有标签
        List<QywxTagGroupDTO> tagGroup = resp.getTagGroup();
        QywxTagGroupDTO groupDTO = tagGroup.get(0);
        //保存企业微信标签信息
        Pair<TabQywxTag, List<TabQywxTagItem>> savePair = qywxTagService.saveQywxTagByQywxGroupKey(wxEnterpriseId, groupDTO);
        if (null == savePair) {
            logger.info("保存失败：{}", JSONObject.toJSONString(groupDTO));
            return;
        }
        //保存关联关系
        saveRelation(wxEnterpriseId, enterpriseId, infoDTO, items, savePair);

        //todo 后续处理 比如同步会员标签等 操作标签项的会员同步过来

    }

    /**
     * 保存标签的关联关系
     *
     * @param wxEnterpriseId
     * @param enterpriseId
     * @param infoDTO
     */
    private void saveRelation(String wxEnterpriseId, String enterpriseId, QywxTagInfoDTO infoDTO, List<QywxTagItemDTO> items, Pair<TabQywxTag, List<TabQywxTagItem>> localQywxTagPair) {
        Map<String, QywxTagItemDTO> orgMap = items.stream().collect(Collectors.toMap(item -> item.getQywxTagName(), dto -> dto));
        //设置关联关系
        TabQywxTag groupKey = localQywxTagPair.getKey();
        List<TabQywxTagItem> tabQywxTagItems = localQywxTagPair.getRight();

        List<TabQywxTagRelation> saveRelationList = new ArrayList<>();
        //如果存在qywxGroupKey 说明已经创建过了
        if (StringUtils.isBlank(infoDTO.getQywxGroupKey())) {
            TabQywxTagRelation groupRelation = new TabQywxTagRelation();
            groupRelation.setEnterpriseId(enterpriseId);
            groupRelation.setMemberTagId(infoDTO.getMemberTagId());
            groupRelation.setQywxTagId(groupKey.getQywxTagId());
            groupRelation.setRelationType(QywxTagRelationTypeEnum.TAG_GROUP.getType());
            groupRelation.setStatusFlag(1);
            groupRelation.setSyncFlag(1);
            groupRelation.setQywxTagRelationId(ToolUtil.randomUUID());
            groupRelation.setWxEnterpriseId(wxEnterpriseId);
            saveRelationList.add(groupRelation);
        }
        List<TabQywxTagRelation> itemList = tabQywxTagItems.stream().map(tab -> {
            QywxTagItemDTO orgItem = orgMap.get(tab.getQywxTagName());
            TabQywxTagRelation itemRelation = new TabQywxTagRelation();
            itemRelation.setWxEnterpriseId(wxEnterpriseId);
            itemRelation.setQywxTagRelationId(ToolUtil.randomUUID());
            itemRelation.setSyncFlag(1);
            itemRelation.setStatusFlag(1);
            itemRelation.setRelationType(QywxTagRelationTypeEnum.TAG_ITEM.getType());
            itemRelation.setQywxTagId(groupKey.getQywxTagId());
            itemRelation.setMemberTagId(infoDTO.getMemberTagId());
            itemRelation.setEnterpriseId(enterpriseId);
            itemRelation.setQywxTagItemId(tab.getQywxTagItemId());
            itemRelation.setTagItemId(orgItem.getQywxTagKey());
            return itemRelation;
        }).collect(Collectors.toList());
        saveRelationList.addAll(itemList);
        qywxTagService.saveQywxRelation(saveRelationList);
    }

    /**
     * 调用企业微信接口 保存标签
     *
     * @param wxEnterpriseDTO
     * @param infoDTO
     * @param items
     * @return
     */
    private QywxGetCorpTagListDTO saveQywxTag(WxEnterpriseDTO wxEnterpriseDTO, QywxTagInfoDTO infoDTO, List<QywxTagItemDTO> items) {
        QywxTagGroupDTO qywxTagGroupDTO = new QywxTagGroupDTO();
        qywxTagGroupDTO.setGroupName("GIC" + infoDTO.getQywxGroupName());
        qywxTagGroupDTO.setOrder(infoDTO.getOrder());
        qywxTagGroupDTO.setGroupId(infoDTO.getQywxGroupKey());

        List<QywxTagDTO> tagItems = items.stream().map(item -> {
            QywxTagDTO qywxTagDTO = new QywxTagDTO();
            qywxTagDTO.setName(item.getQywxTagName());
            qywxTagDTO.setOrder(item.getOrder());
            qywxTagDTO.setId(item.getQywxTagKey());
            return qywxTagDTO;
        }).collect(Collectors.toList());
        qywxTagGroupDTO.setTag(tagItems);

        return qywxSuiteApiService.saveCorpTag(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), qywxTagGroupDTO);
    }

    @Override
    public void closeSync(String wxEnterpriseId, String memberTagId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        List<TabQywxTagRelation> tagRelations = qywxTagService.listQywxTagRelationByMemberTagId(wxEnterpriseId, memberTagId);
        if (CollectionUtils.isEmpty(tagRelations)) {
            return;
        }
        TabQywxTag tabQywxTag = qywxTagService.getQywxTagByQywxTagId(wxEnterpriseId, tagRelations.get(0).getQywxTagId());
        //关闭同步
        qywxTagService.closeSync(wxEnterpriseId, memberTagId);
        List<String> delIds = new ArrayList<>();
        delIds.add(memberTagId);
        QywxResponseDTO responseDTO = qywxSuiteApiService.delCorpTag(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid(), delIds, 0);
        if (responseDTO.getErrcode() == 0) {
            qywxTagService.delQywxTag(tabQywxTag.getQywxGroupKey(), wxEnterpriseId);
        }

    }

    @Override
    public List<QywxTagRelationDTO> listAllByWxEnterpriseId(String wxEnterpriseId) {
        List<TabQywxTagRelation> ret = qywxTagService.listAllQywxRelation(wxEnterpriseId);
        return EntityUtil.changeEntityListByJSON(QywxTagRelationDTO.class, ret);
    }

    @Override
    public void addQywxTag(QywxTagCallbackDTO callbackDTO) {
        logger.info("企业微信新增标签回调：{}", JSONObject.toJSONString(callbackDTO));
        if (!config.getWxSuiteid().equals(callbackDTO.getSuiteId())) {
            logger.info("非第三方回调");
            return;
        }
        TabHaobanWxEnterprise wxEnterpriseDto = wxEnterpriseService.getEnterpriseBycorpId(callbackDTO.getAuthCorpId());
        if (null == wxEnterpriseDto) {
            logger.info("企业不存在：{}", callbackDTO.getAuthCorpId());
            return;
        }
        QywxTagRelationTypeEnum relationTypeEnum = QywxTagRelationTypeEnum.getByTagType(callbackDTO.getTagType());
        //标签组
        if (relationTypeEnum.getTagType() == QywxTagRelationTypeEnum.TAG_GROUP.getTagType()) {
            TabQywxTag qywxGroupKey = qywxTagService.getQywxTagByQywxGroupKey(wxEnterpriseDto.getWxEnterpriseId(), callbackDTO.getTagId());
            if (null == qywxGroupKey) {
                logger.info("原型不存在，不需要处理");
                return;
            }
            List<TabQywxTagRelation> relations = qywxTagService.listQywxTagRelationByQywxTagId(wxEnterpriseDto.getWxEnterpriseId(), qywxGroupKey.getQywxTagId());
            //没有同步的关系 删除保存企业微信标签
            if (CollectionUtils.isEmpty(relations)) {
                logger.info("删除保存的企业微信标签");
                qywxTagService.delQywxTag(callbackDTO.getTagId(), wxEnterpriseDto.getWxEnterpriseId());
                return;
            }
            List<String> groupIds = new ArrayList<>();
            groupIds.add(callbackDTO.getTagId());
            qywxSuiteApiService.delCorpTag(wxEnterpriseDto.getCorpid(), config.getWxSuiteid(), groupIds, QywxTagRelationTypeEnum.TAG_GROUP.getType());
            //标签项
        } else if (relationTypeEnum.getTagType() == QywxTagRelationTypeEnum.TAG_ITEM.getTagType()) {
            TabQywxTagItem qywxTagItem = qywxTagService.getQywxTagItemByQywxItemKey(wxEnterpriseDto.getWxEnterpriseId(), callbackDTO.getTagId());
            if (null == qywxTagItem) {
                logger.info("没有关联，无需操作");
                return;
            }
            List<TabQywxTagRelation> relations = qywxTagService.listQywxTagRelationByMemberTagId(wxEnterpriseDto.getWxEnterpriseId(), qywxTagItem.getQywxTagId());
            if (CollectionUtils.isEmpty(relations)) {
                logger.info("直接删除标签项");
                qywxTagService.delQywxTagItemById(wxEnterpriseDto.getWxEnterpriseId(), qywxTagItem.getQywxTagItemId());
                return;
            }
            List<String> items = new ArrayList<>();
            items.add(callbackDTO.getTagId());
            qywxSuiteApiService.delCorpTag(wxEnterpriseDto.getCorpid(), config.getWxSuiteid(), items, QywxTagRelationTypeEnum.TAG_ITEM.getType());
        }
    }

    @Override
    public void delQywxTag(QywxTagCallbackDTO callbackDTO) {
        logger.info("企业微信删除标签回调：{}", JSONObject.toJSONString(callbackDTO));
        if (!config.getWxSuiteid().equals(callbackDTO.getSuiteId())) {
            logger.info("非第三方回调");
            return;
        }
        TabHaobanWxEnterprise wxEnterpriseDto = wxEnterpriseService.getEnterpriseBycorpId(callbackDTO.getAuthCorpId());
        if (null == wxEnterpriseDto) {
            logger.info("企业不存在：{}", callbackDTO.getAuthCorpId());
            return;
        }
        QywxTagRelationTypeEnum relationTypeEnum = QywxTagRelationTypeEnum.getByTagType(callbackDTO.getTagType());
        //标签组
        if (relationTypeEnum.getTagType() == QywxTagRelationTypeEnum.TAG_GROUP.getTagType()) {
            TabQywxTag qywxGroupKey = qywxTagService.getQywxTagByQywxGroupKey(wxEnterpriseDto.getWxEnterpriseId(), callbackDTO.getTagId());
            if (null == qywxGroupKey) {
                logger.info("原型不存在，不需要处理");
                return;
            }
            List<TabQywxTagRelation> relations = qywxTagService.listQywxTagRelationByQywxTagId(wxEnterpriseDto.getWxEnterpriseId(), qywxGroupKey.getQywxTagId());
            //没有同步的关系 删除保存企业微信标签
            if (CollectionUtils.isEmpty(relations)) {
                logger.info("删除保存的企业微信标签");
                qywxTagService.delQywxTag(callbackDTO.getTagId(), wxEnterpriseDto.getWxEnterpriseId());
                return;
            }
            //存在同步关系 需要回写回去
            List<TabQywxTagItem> tabQywxTagItems = qywxTagService.listByQywxTagId(wxEnterpriseDto.getWxEnterpriseId(), qywxGroupKey.getQywxTagId());
            this.reSyncTagGroupBack(wxEnterpriseDto.getWxEnterpriseId(), relations, qywxGroupKey, tabQywxTagItems);
            //标签项
        } else if (relationTypeEnum.getTagType() == QywxTagRelationTypeEnum.TAG_ITEM.getTagType()) {
            TabQywxTagItem qywxTagItem = qywxTagService.getQywxTagItemByQywxItemKey(wxEnterpriseDto.getWxEnterpriseId(), callbackDTO.getTagId());
            if (null == qywxTagItem) {
                logger.info("没有关联，无需操作");
                return;
            }
            List<TabQywxTagRelation> relations = qywxTagService.listQywxTagRelationByMemberTagId(wxEnterpriseDto.getWxEnterpriseId(), qywxTagItem.getQywxTagId());
            if (CollectionUtils.isEmpty(relations)) {
                logger.info("直接删除标签项");
                qywxTagService.delQywxTagItemById(wxEnterpriseDto.getWxEnterpriseId(), qywxTagItem.getQywxTagItemId());
                return;
            }
            //同步
            reSyncQywxTagItem(wxEnterpriseDto.getWxEnterpriseId(), relations, qywxTagItem);

        }

    }

    /**
     * 重新同步
     *
     * @param wxEnterpriseId
     * @param relations
     * @param qywxGroupKey
     * @param tabQywxTagItems
     */
    private void reSyncTagGroupBack(String wxEnterpriseId, List<TabQywxTagRelation> relations, TabQywxTag qywxGroupKey, List<TabQywxTagItem> tabQywxTagItems) {
        TabQywxTagRelation tagRelation = relations.get(0);
        String enterpriseId = tagRelation.getEnterpriseId();
        String memberTagId = tagRelation.getMemberTagId();
        //关闭同步
        qywxTagService.closeSync(wxEnterpriseId, memberTagId);
        qywxTagService.delQywxTag(qywxGroupKey.getQywxGroupKey(), wxEnterpriseId);

        //重新同步
        Map<String, TabQywxTagRelation> relationMap = relations.stream().collect(Collectors.toMap(dto -> {
            if (dto.getRelationType() == QywxTagRelationTypeEnum.TAG_GROUP.getType()) {
                return dto.getQywxTagId();
            } else {
                return dto.getQywxTagItemId();
            }
        }, dto -> dto));

        QywxTagInfoDTO qywxTagInfoDTO = new QywxTagInfoDTO();
        qywxTagInfoDTO.setQywxGroupKey(memberTagId);
        qywxTagInfoDTO.setQywxGroupName(qywxGroupKey.getQywxGroupName());
        qywxTagInfoDTO.setWxEnterpriseId(qywxGroupKey.getWxEnterpriseId());

        List<QywxTagItemDTO> items = tabQywxTagItems.stream().map(item -> {
            TabQywxTagRelation relation = relationMap.get(item.getQywxTagItemId());
            QywxTagItemDTO mid = new QywxTagItemDTO();
            mid.setQywxTagKey(relation.getTagItemId());
            mid.setQywxTagName(item.getQywxTagName());
            return mid;
        }).collect(Collectors.toList());
        //同步回去
        this.syncTagToQywx(wxEnterpriseId, enterpriseId, qywxTagInfoDTO, items);
    }

    /**
     * 同步标签项
     *
     * @param wxEnterpriseId
     * @param qywxTagItem
     */
    private void reSyncQywxTagItem(String wxEnterpriseId, List<TabQywxTagRelation> relations, TabQywxTagItem qywxTagItem) {
        //删除标签
        qywxTagService.delQywxTagItemById(wxEnterpriseId, qywxTagItem.getQywxTagItemId());

        //同步标签项
        TabQywxTag qywxGroupKey = qywxTagService.getQywxTagByQywxGroupKey(wxEnterpriseId, qywxTagItem.getQywxGroupKey());
        //各自关联关系
        TabQywxTagRelation groupRelation = relations.stream().filter(dto -> dto.getRelationType() == QywxTagRelationTypeEnum.TAG_GROUP.getType()).findFirst().get();
        TabQywxTagRelation itemRelation = relations.stream().filter(dto -> (dto.getRelationType() == QywxTagRelationTypeEnum.TAG_ITEM.getType() && qywxTagItem.getQywxTagItemId().equals(dto.getQywxTagItemId()))).findFirst().orElse(null);

        QywxTagInfoDTO qywxTagInfoDTO = new QywxTagInfoDTO();
        qywxTagInfoDTO.setWxEnterpriseId(wxEnterpriseId);
        qywxTagInfoDTO.setMemberTagId(groupRelation.getMemberTagId());
        qywxTagInfoDTO.setQywxGroupKey(qywxGroupKey.getQywxGroupKey());
        qywxTagInfoDTO.setOrder(qywxGroupKey.getOrder());

        List<QywxTagItemDTO> itemList = new ArrayList<>();
        QywxTagItemDTO qywxTagItemDTO = new QywxTagItemDTO();
        qywxTagItemDTO.setMemberTagItemId(itemRelation.getTagItemId());
        qywxTagItemDTO.setQywxTagName(qywxTagItem.getQywxTagName());
        qywxTagItemDTO.setWxEnterpriseId(wxEnterpriseId);
        qywxTagItemDTO.setOrder(qywxTagItem.getOrder());
        itemList.add(qywxTagItemDTO);
        //同步
        this.syncTagToQywx(wxEnterpriseId, groupRelation.getEnterpriseId(), qywxTagInfoDTO, itemList);


    }

    @Override
    public void editQywxTag(QywxTagCallbackDTO callbackDTO) {
        logger.info("企业微信修改标签回调：{}", JSONObject.toJSONString(callbackDTO));
        if (!config.getWxSuiteid().equals(callbackDTO.getSuiteId())) {
            logger.info("非第三方回调");
            return;
        }
        TabHaobanWxEnterprise wxEnterpriseDto = wxEnterpriseService.getEnterpriseBycorpId(callbackDTO.getAuthCorpId());
        if (null == wxEnterpriseDto) {
            logger.info("企业不存在：{}", callbackDTO.getAuthCorpId());
            return;
        }
        QywxTagRelationTypeEnum relationTypeEnum = QywxTagRelationTypeEnum.getByTagType(callbackDTO.getTagType());
        //标签组
        if (relationTypeEnum.getTagType() == QywxTagRelationTypeEnum.TAG_GROUP.getTagType()) {
            TabQywxTag qywxGroupKey = qywxTagService.getQywxTagByQywxGroupKey(wxEnterpriseDto.getWxEnterpriseId(), callbackDTO.getTagId());
            if (null == qywxGroupKey) {
                logger.info("原型不存在，不需要处理");
                return;
            }
            List<TabQywxTagRelation> relations = qywxTagService.listQywxTagRelationByQywxTagId(wxEnterpriseDto.getWxEnterpriseId(), qywxGroupKey.getQywxTagId());
            //没有同步的关系 删除保存企业微信标签
            if (CollectionUtils.isEmpty(relations)) {
                logger.info("删除保存的企业微信标签");
                qywxTagService.delQywxTag(callbackDTO.getTagId(), wxEnterpriseDto.getWxEnterpriseId());
                return;
            }
            //名称同步回去
            QywxTagDTO qywxTagDTO = new QywxTagDTO();
            qywxTagDTO.setId(callbackDTO.getTagId());
            qywxTagDTO.setName(qywxGroupKey.getQywxGroupName());
            qywxTagDTO.setOrder(qywxGroupKey.getOrder());
            qywxSuiteApiService.editCorpTagNameOrOrder(wxEnterpriseDto.getCorpid(), config.getWxSuiteid(), qywxTagDTO);

            //标签项
        } else if (relationTypeEnum.getTagType() == QywxTagRelationTypeEnum.TAG_ITEM.getTagType()) {
            TabQywxTagItem qywxTagItem = qywxTagService.getQywxTagItemByQywxItemKey(wxEnterpriseDto.getWxEnterpriseId(), callbackDTO.getTagId());
            if (null == qywxTagItem) {
                logger.info("没有关联，无需操作");
                return;
            }
            List<TabQywxTagRelation> relations = qywxTagService.listQywxTagRelationByMemberTagId(wxEnterpriseDto.getWxEnterpriseId(), qywxTagItem.getQywxTagId());
            if (CollectionUtils.isEmpty(relations)) {
                logger.info("直接删除标签项");
                qywxTagService.delQywxTagItemById(wxEnterpriseDto.getWxEnterpriseId(), qywxTagItem.getQywxTagItemId());
                return;
            }
            //名称同步回去
            QywxTagDTO qywxTagDTO = new QywxTagDTO();
            qywxTagDTO.setId(callbackDTO.getTagId());
            qywxTagDTO.setName(qywxTagItem.getQywxTagName());
            qywxTagDTO.setOrder(qywxTagItem.getOrder());
            qywxSuiteApiService.editCorpTagNameOrOrder(wxEnterpriseDto.getCorpid(), config.getWxSuiteid(), qywxTagDTO);

        }
    }
}
